/*************************************************************************
*                                                                        *
*  Name : fadapt.c                                                       *
*                                                                        *
*  Purpose : Driver program of the ADAPTOR translation tool              *
*                                                                        *
*  Author : Dr. Thomas Brandes, GMD, I1.HR                               *
*                                                                        *
*  Last Update : June 1993                                               *
*                                                                        *
*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "global.h"
#include "Parser.h"
#include "Tree.h"
#include "MakeDefs.h"
#include "Analysi.h"
#include "Temporar.h"
#include "Init.h"
#include "Serial.h"
#include "Transfor.h"
#include "Definiti.h"
#include "Unparse.h"
#include "CallGraF.h"     /* for TheCallGraph */
#include "Calling.h"
#include "DataFlow.h"

# ifdef alliant
# include <fcntl.h>		/* for testing fileaccess */
# else
# include <unistd.h>		/* for testing fileaccess */
# endif

#include <sys/types.h>		/* for getpid */

/* external variables defined in this module */

int interactive = 0;
char ** fad_argv;     /* used for makefile writer */
int     fad_argc;     /* used for makefile writer */

#define DEBUG(x)
#define MAXCOMMANDLINELENGTH 1024
#define MAXPATHNAMELENGTH 1024
#define USETMP 2
#define TMP "/tmp/adaptor"

static int usetmp;
static char cat[MAXCOMMANDLINELENGTH];
static char tempfile[MAXCOMMANDLINELENGTH];

/****************************************************************/
/* is_source_file()						*/
/* INPUT:	string						*/
/*		length (length of string)			*/
/* OUTPUT:	true, if string is an allowed fortran filename	*/
/*		false, if string isn`t a fortran filename	*/
/****************************************************************/

bool is_source_file (string, length)
char *string;
int length;
{
   if (strcmp(string+length-2,".f") == 0) return (true);
   if (strcmp(string+length-2,".F") == 0) return (true);
   if (strcmp(string+length-3,".f9") == 0) return (true);
   if (strcmp(string+length-3,".F9") == 0) return (true);
   if (strcmp(string+length-4,".fcm") == 0) return (true);
   return(false);
}


int SourceParser (fname)
char *fname;

/* parses the fortran file 'fname', returns number
   of syntax errors in the source file
   -1 : file could not be opened                       */

{  int Errors;

   filename = fname;    /* for makefile write, will know the file name */
   BeginFile (fname);
   BeginPos (fname);
   Errors = Parse ();
   ClosePos ();
   if (Errors == 0)
       sprintf (last_message, "Succesful parsed");
     else
       sprintf (last_message, "Parsing with %d Errors", Errors);
   return (Errors);
}

int SourceCheck ()
{  if (!CheckTree (TreeRoot))
     { sprintf (last_message, "Tree is not okay");
       return 1;
     }
    else
     { sprintf (last_message, "Tree is good");
       return 0;
     }
}

int SourceSemantic ()

{ int phase;
  int errors;

  printf ("\n");
  printf ("SEMANTIC PHASE 1 : Make Definitions\n");
  phase = 1;
  MakeDefs (TreeRoot);
  errors = protocol_errors ();
  if (errors > 0)
    { printf (" %d Errors occured, see adaptor.def\n", errors);
      sprintf (last_message,
         "%d Errors in SEMANTIC 1 : MakeDefs (see adaptor.def)", errors);
         goto Ende;
    }
  printf (" successfully completed\n");

  printf ("\n");
  printf ("SEMANTIC PHASE 2 : Checking\n");
  phase = 2;
  BeginSemantic ();
  Semantic (TreeRoot);
  CloseSemantic ();
  errors = protocol_errors ();
  if (errors > 0)
    { printf (" %d Errors occured, see adaptor.sem\n", errors);
      sprintf (last_message,
         "%d Errors in SEMANTIC 2 : Checks (see adaptor.sem)", errors);
      goto Ende;
    }
  printf (" successfully completed\n");

  printf ("\n");
  printf ("SEMANTIC PHASE 3 : Control Flow\n");
  phase = 3;
  ControlFlow (TreeRoot);
  errors = protocol_errors ();
  if (errors > 0)
    { printf (" %d Errors occured, see adaptor.cf\n", errors);
      sprintf (last_message,
         "%d Errors in SEMANTIC 3 : Control Flow (see adaptor.cf)", errors);
      goto Ende;
    }
  printf (" successfully completed\n");

  sprintf (last_message, "Semantic Analysis was successful");
  phase = 0;

Ende: return (phase);
}

int SourceWrite ()

{  FILE *myFile;
   printf ("Write Tree on File test.out \n");
   myFile = fopen ("test.out","w");
   if (myFile == (FILE *) NULL)
    { printf ("Adaptor failed to open file test.out\n");
      printf ("Please check permissions\n");
      exit (-1);
    }
   WriteTree (myFile, TreeRoot);
   fclose (myFile);
   printf ("Writing Tree is ready\n");
   sprintf (last_message, "Abtract Tree written to file test.out");
}

int SourceCalling ()

{  FILE *myFile;

   BeginCalling ();
   printf ("Starting Calling Analysis \n");
   Calling (TreeRoot);
   printf ("Ending Calling Analysis\n");

   /* printf ("Write CallGraph on File test.cg \n");
      myFile = fopen ("test.cg","w");
      if (myFile == (FILE *) NULL)
       { printf ("Adaptor failed to open file test.cg\n");
         printf ("Please check permissions\n");
         exit (-1);
       }
      WriteCallGraph (myFile, TheCallGraph);
      fclose (myFile);
      printf ("Writing CallGraph is ready\n");
      CloseCalling ();
   */

   /* import CGFile from Calling.h */
   printf ("Start Writing CallGraph \n");
   CGFile = fopen ("test.cal","w");
   if (CGFile == (FILE *) NULL)
    { printf ("Adaptor failed to open file test.cal\n");
      printf ("Please check permissions\n");
      exit (-1);
    }
   OutCallGraph (TheCallGraph);
   fclose (CGFile);
   printf ("End Writing CallGraph to a File\n");
   sprintf (last_message, "Call Graph has been generated");
}

int SourceDataFlow ()

{  BeginDataFlow ();
   printf ("Starting Data Flow Analysis \n");
   DataFlow (TreeRoot);
   printf ("Ending Data Flow Analysis\n");
   CloseDataFlow ();

   printf ("Start Writing Dependences \n");
   DepFile = fopen ("test.dp","w");
   if (DepFile == (FILE *) NULL)
    { printf ("Adaptor failed to open file test.dp\n");
      printf ("Please check permissions\n");
      exit (-1);
    }
   OutDependences (TreeRoot);
   fclose (DepFile);
   printf ("End Writing Dependences to a File\n");
}

void SourceUnparse ()

{ BeginUnparse ();
  Unparse (TreeRoot);
  CloseUnparse ();
}

int SourceAdapt ()
{  FILE *f;
   tTree HostRoot;
   int errors;
   int phase;
   char node_file [10];  /* name of file for node/cube/node1 program */

   /* PHASE 1 : ADAPTOR Analysis */

     printf ("\n");
     printf ("ADAPTOR PHASE 1 : Analysis\n");
     phase = 1;
     BeginAdaptAnalysis ();
     AdaptAnalysis (TreeRoot);
     CloseAdaptAnalysis ();
     errors = protocol_errors ();
     if (errors > 0)
       { printf (" %d Errors occured, see adaptor.anl\n", errors);
         sprintf (last_message,
            "%d Errors in PHASE 1 : Analysis (see adaptor.anl)", errors);
         goto Ende;
       }
     printf (" successfully completed\n");

   /* PHASE 2 : ADAPTOR Distributions */

     printf ("\n");
     printf ("ADAPTOR PHASE 2 : Distributions\n");
     phase = 2;
     BeginAdaptDistributions ();
     AdaptDistributions (TreeRoot);
     CloseAdaptDistributions ();
     errors = protocol_errors ();
     if (errors > 0)
       { printf (" %d Errors occured, see adaptor.dis\n", errors);
         sprintf (last_message,
            "%d Errors in PHASE 2 : Distributions (see adaptor.dis)", errors);
         goto Ende;
       }
     printf (" successfully completed\n");

   /* PHASE 3 : ADAPTOR Temporary */

     printf ("\n");
     printf ("ADAPTOR PHASE 3 : Temporary Variables \n");
     phase = 3;
     AdaptTemporary (TreeRoot);
     errors = protocol_errors ();
     if (errors > 0)
       { printf (" %d Errors occured, see adaptor.tmp\n", errors);
         sprintf (last_message,
            "%d Errors in PHASE 3 : Temporary (see adaptor.tmp)", errors);
         goto Ende;
       }
     if (!CheckTree (TreeRoot))
       { printf ("Tree is not okay after creating temporaries \n");
         sprintf (last_message, "Illegal Tree after PHASE 3 : Temporary ");
         goto Ende;
       }
     printf (" successfully completed\n");

   /* PHASE 4 : ADAPTOR Initialization */

     printf ("\n");
     printf ("ADAPTOR PHASE 4 : Initial\n");
     phase = 4;
     AdaptInit (TreeRoot);
     errors = protocol_errors ();
     if (errors > 0)
       { printf (" %d Errors occured, see adaptor.ini\n", errors);
         sprintf (last_message,
            "%d Errors in PHASE 4 : Initial (see adaptor.ini)", errors);
         phase = 4;
         goto Ende;
       }
     if (!CheckTree (TreeRoot))
       { printf ("Tree is not okay after initial transformation\n");
         sprintf (last_message, "Illegal Tree after PHASE 3 : Initial ");
         goto Ende;
       }
     printf (" successfully completed\n");

   /* PHASE 5 : ADAPTOR Serialization */

     printf ("\n");
     printf ("ADAPTOR PHASE 5 : Serial\n");
     phase = 5;
     AdaptSerial (TreeRoot);
     errors = protocol_errors ();
     if (errors > 0)
       { printf (" %d Errors occured, see adaptor.seq\n", errors);
         sprintf (last_message,
            "%d Errors in PHASE 5 : Serial (see adaptor.seq)", errors);
         phase = 5;
         goto Ende;
       }
     if (!CheckTree (TreeRoot))
       { printf ("Tree is not okay after serialization\n");
         sprintf (last_message, "Illegal Tree after PHASE 5 : Serial ");
         goto Ende;
       }
     SourceUnparse ();
     printf (" successfully completed (see unparse.f)\n");

   /* PHASE 6 : ADAPTOR Transformation */

     printf ("\n");
     printf ("ADAPTOR PHASE 6 : Transform\n");
     phase = 6;

     if (target_model == HOST_NODE)
     { HostRoot = CopyTree (TreeRoot);
       HostAdapt (HostRoot);
       errors = protocol_errors ();
       if (errors > 0)
         { printf (" %d Errors occured, see adaptor.tra\n", errors);
           sprintf (last_message,
              "%d Errors in PHASE 5 : Host (see adaptor.tra)", errors);
           goto Ende;
         }
       if (!CheckTree (HostRoot))
         { printf ("Host Tree is not okay\n");
           sprintf (last_message,"Illegal Host Tree after PHASE 5 : Transform");
           goto Ende;
         }

       f = fopen ("host.f", "w");
       if (f == (FILE *) NULL)
        { printf ("Adaptor failed to open file host.f\n");
          printf ("Please check permissions\n");
          exit (-1);
        }
       FileUnparse (f, HostRoot);
       fclose (f);
       printf (" Host program successfully generated\n");
     }

     /* define node_file */

     if (target_model == HOST_NODE)
        sprintf (node_file, "node.f");
      else if (target_model == UNI_PROC)
        sprintf (node_file, "node1.f");
      else /* target_model == ONLY_NODE */
        sprintf (node_file, "cube.f");

     NodeAdapt (TreeRoot);
     errors = protocol_errors ();
     if (errors > 0)
       { printf (" %d Errors occured, see adaptor.tra\n", errors);
         sprintf (last_message,
            "%d Errors in PHASE 6 : Node (see adaptor.tra)", errors);
         goto Ende;
       }
     if (!CheckTree (TreeRoot))
       { printf ("Node Tree is not okay\n");
         sprintf (last_message, "Illegal Node Tree after PHASE 5 : Transform");
         goto Ende;
       }

     f = fopen (node_file, "w");
     if (f == (FILE *) NULL)
      { printf ("Adaptor failed to open file %s\n", node_file);
        printf ("Please check permissions\n");
        exit (-1);
      }
     FileUnparse (f, TreeRoot);
     fclose (f);
     printf (" Node program (%sRoot);
 Tree is n
     if (errors > 0)
 enntf tarermis  if (erHASE 6 :        exi  if (errn", node_file);)
 enntf   rmis  if (erHASE 6 : (last_mess    { printf ("Node;
     er is not okay\cg ULL)
      { sprintf (last_messagecgIllegal Nodeode Tree after PHASE 5 : Transform");");
         goto Ende;
       }

     f = cgULL)
      {ile, "w");
     if (f == (FILE *) NULL)
      {  { printf ("Adaptoptor failailed to;
     er file %s\nt);
     e("Adaptopt
        printf ("Pl"Please check permis;
     er\n");
        exi f ("Ada  if (errors > 0ptor.sss    i3 : t CGafterfrom l_errors (TreeRoo)
 enntf tare k permis;
     er       exi Gprintf (last_messagecalIllegal Node Tre GprintfPHASE 5 : Transform");
         goto Ende;
       }

     f = calen (node_file, "w");
     if (f == (FILE *) NULL)
      { printf ("Adaptor failOut;
     er nt);
     e("Adapt
       Gprin_file);)
 enntf    k permis;
     erOut=s not       exit (-1);
      }
     F;
      ernderor

gn Prat goto E;
     printf (errors  program (
   (errors  prrs > 0)
 enntf tarermis(errors > HASE 6 :        exi(errors  pn", node_file);)
 enntf   rmis(errors > HASE 6 :       exi     (errors  prrs eeRoo)
 enntf tare k permisDe}
denee         exi(epprintf (last_messagedpIllegal Node Tre(epprintfPHASE 5 : Transform");
         goto Ende;
       }

     f = dpen (node_file, "w");
     if (f == (FILE *) NULL)
      { printf ("Adaptor failOutDe}
denee  (n", node_file);
      (epprin_file);)
 enntf    k permisDe}
denee  Out=s not       ;
  voidprintf 
     p program 
   
     p prfile)
     p p     { printf ("Ada
     p prfil;
     printf goto ADAPTOR SE 5 :ffile);tse (fHost { pl Node col_errors (ode colors = protial\n"nodis not [10];s    ree (of
     gotnodi/cube/nodi1 (see adaptor.sss    rintf (" %l)", errHASE 6 : ptor.sss HOST_NODE)
         HOST_NODE)
l)", errrintf (" %lASE 6 :       exidaptor.tra\n", es n
   goto HASE 6 : (    exidagoto HASE 6 : (n", node_file);f ("Adagoto HASE 6 : (    exida(node_file, "node1.f");
      e  else /* target_model =  == ONLY_NODE */
        sprintf (node_file,anlcube.f");

     Nod odeAdapt (TreeRoot);
     errors =  = protocol_errorintf (" %lASE 6 : 0)
       { anln");
           sprintf (last_message,ptor failaia\n", errors);
         sprintf (last_mess    rintf 2" %l)", errDi");
bucured ptor.sss HOST_NODE)
         HOST_NODE)
l)", errrintf 2" %Di");
bucured       exidaptor.tra2n", es n
   goto Di");
bucured (    exidagoto Di");
bucured (n", node_file);f ("Adagoto Di");
bucured (    exida(node_file, "node1.f");
      e  else /* target_model =  == ONLY_NODE */
        sprintf (node_file,d :   e.f");

     Nod odeAdapt (TreeRoot);
     errors =  = protocol_errorintf 2" %Di");
bucuredTree is not okaisn");
           sprintf (last_message,ptor failaia\n", errors);
         sprintf (last_mess    rintf 3" %l)", errE 3 : ADy ptor.sss HOST_NODE)
         HOST_NODE)
l)", errrintf 3" %E 3 : ADy V
       }       exidaptor.tra3   exidagoto E 3 : ADy (n", node_file);f (node_file, "node1.f");
      e  else /* target_model =  == ONLY_NODE */
        sprintf (node_file,tmp   e.f");

     Nod odeAdapt (TreeRoot);
     errors =  = protocol_errorintf 3" %E 3 : ADy ( (  OI    N3" %E  n
    N   spONLY__       er fi   spONLY__)
aultsASE 2 : AD n
   goprintf  e;
     ach enn failer (fnam_;
     ach en;f  e;
      else /* ailer (fnam_;
      else /;f  e;
       }
 n failer (fnam_;
       }
;f  e ONLY_NODE  
 n failer (fnam_ ONLY_NODE;f  ed)
ault_NODE  
failer (fnam_d)
ault_NODE;f  erors);








failer (fnam_rors);
;f  era3   exidagoto failer (fnam_ra3   exidagoto;f  esplit_flag





failer (fnam_split_flagintf  e_mespyoto HASE iler (fnam_errDi");
ed (n",errDiroto_errreplacint  exi  euTempoenvironme  OourcePar "node1.f");
   envto HAS_envoure GprintfP_mespyto HASE   envto HAS_envoure );
ed (e', retu
fa : ADy e file
  
fa : c);
ed (os (fngoprinttf (last_(rrors ==,   %d",TMP,urn(fa())inttf (last_(cat,"cat >%s",rrors ==)inttfi=ocol_ewh occui< : c : ADy (n n

fa : A[i    n",-oo)
 pto)
aultewh(-1us (strcmp, es pr prigume  ULL)
      ");
'-'     n
[0]de_file);
   Nod odeAdapt (Tn n

fa"-oo)
";  Nod odeAdapt (Tnos (fn++;  Nod odeAdapt (Tn");
os (fn< the f) ();
  erro : A[i    Nod odeAdapt (Tnoto E();
  errorrors ==;  Nod odeAdapt (Tn");

bucur( : A[i 
         HOST_NODE)
l)le);
   Nod odeAdapt (TnnnnnnnnnAdagoto HASE 6 : (    exida(n%s!     : A[i )int("Aram 
   (e		}(e		 %d Er		{nt("A");
"Parsing with % : A[i 
_me;
% : A[i )s in ;

 nt("A{nt("A	Adagoto HASE 6 : ( F;
     }
    f (erro	("Aram 
   (e			}(e			_mespytcat+_me;
%cat)-_me;
%rrors ==)-1  : A[i )int("A_mespytcat+_me;
%cat)," >")int("A_mespytcat+_me;
%cat),rrors ==)int		}(e	}
}

DE mpo-oo)
 pto)
aultewh(-1us (strcmp, es pr prigume  gULL)
    last_messag n
,"-nntf rs in SEde_file);f (" n
   gopocol	  printf (ntf (last_messag n
,"-sCGFile rs in SEde_file);f (" n
   gop2col	  printf (ntf (last_messag n
,"-cto rs in SEde_file);f (" n
   gop3col	  printf (ntf (last_messag n
,"-oo)
"s in SEde_file);f (" n
   gop4col	  printf (ntf (last_messag n
,"-p"s in SEde_file);f (  Nod odeAdapt ");

  
>=;
"+1) =  ==ssotof % : A[i+1]
   d", &rors);
))HOST_NODE)
l)le)	i++;  	ntf (ntf   	ntf ({nt("Adagoto HASE 6 :rigume  g   pr(stDy rigume  gn (t wrongf (erro	( errrintf 2" %Di"eer ould nrigume  g-pf (erro	(ram 
   (e	ntf (}OST_NODE)
l printf (ntf (last_messag n
,"-F90"s in SEde_file);f  e;
      else /* arget_model;printf (ntf (last_messag n
,"-F77"s in SEde_file);f  e;
      else /* arget_mod77;printf (ntf (last_messag n
,"-S"s in SEde_file);f  e{e ONLY_NODE /
        spri;  Nod odeAdapt ");

  
>;
"+1) =  ro	(=ssotof % : A[i+1]
   d", &ra3   exidagoto))HOST_NODE)
l)le)	i++;  	ntf (ntf   	ntf ({nt("Adagoto HASE 6 :rigume  g   pr(stDy rigume  gn (t wrongf (erro	( errrintf 2" %Di"eer ould nrigume  g-Sf (erro	(ram 
   (e	ntf (}OST_NODE)
l
l printf (ntf (last_messag n
,"-D"s in SEde_file);f  e ONLY_NODE /
DYNAM     spri;  Nod odntf (last_messag n
,"-sun"s in SEde_file);f  e;
     ach enn/
UN4_PVM;  Nod odntf (last_messag n
,"-os2pvm"s in SEde_file);f  e;
     ach enn/
OS2_PVM;  Nod odntf (last_messag n
,"-ibm"s in SEde_file);f  e;
     ach enn/
RIOS_PVM;  Nod odntf (last_messag n
,"-ength-3"s in SEde_file);f  e;
     ach enn/
ALLIANT;  Nod odntf (last_messag n
,"-eng_pvm"s in SEde_file);f  e;
     ach enn/
ALL_PVM;  Nod odntf (last_messag n
,"-ipsc"s in SEde_file);f  e;
     ach enn/
IPSC_86l;printf (ntf (last_messag n
,"-gc"s in SEde_file);f  e;
     ach enn/
PARSYTEC_GC;printf (ntf (last_messag n
,"-meiko_cs1"s in SEde_file);f  e;
     ach enn/
MEIKO_CS1;printf (ntf (last_messag n
,"-meiko_cs2"s in SEde_file);f  e;
     ach enn/
MEIKO_CS2;printf (ntf (last_messag n
,"-ksr1"s in SEde_file);f  e;
     ach enn/
KSR1;printf (ntf (last_messag n
,"-ksr"s in SEde_file);f  e;
     ach enn/
KSR1;printf (ntf (last_messag n
,"-ksr1_pvm"s in SEde_file);f  e;
     ach enn/
KSR1_PVM;  Nod odntf (last_messag n
,"-ksr_pvm"s in SEde_file);f  e;
     ach enn/
KSR1_PVM;  Nod odntf (last_messag n
,"-cm5"s in SEde_file);f  e;
     ach enn/
CM5;  Nod odntf (last_messag n
,"-cm"s in SEde_file);f  e;
     ach enn/
CM5;  Nod odntf (last_messag n
,"-sgi"s in SEde_file);f  e;
     ach enn/
GI;  Nod odntf (last_messag n
,"-1"s in SEde_file);f  e;
       }

al Nodeod;  Nod odntf (last_messag n
,"-uniE 3 "s in SEde_file);f  e;
       }

al Nodeod;  Nod odntf (last_messag n
,"-split"s in SEde_file);f  esplit_flag
otocol_errodntf (last_messag n
,"-nosplit"s in SEde_file);f  esplit_flag
otl;printf (ntf (last_messag n
,"-ddr"s in SEde_file);f  ed)
ault_NODE /
DDEFAULT_REPLICATED;printf (ntf (last_messag n
,"-ddb"s in SEde_file);f  ed)
ault_NODE /
DDEFAULT_DISTRIBUTED;printf (ntf (last_messag n
,"-ddcm"s in SEde_file);f  ed)
ault_NODE /
DDEFAULT_CM;  Nod odntf (last_messag n
,"-N"s in SEde_file);f  e;
       }

a    f = c;  Nod odntf (last_messag n
,"-H"s in SEde_file);f  e;
       }

a printf (;  Nod odntf (last_messag n
,"-h
   s in SEde_file);f (  Nod odeAdapt ");

  
>;
"+1) =  ==ssotof % : A[i+1]
   s", errDi")HOST_NODE)
l)le)	i++;  	ntf (ntf   	ntf ({nt("Adagoto HASE 6 :rigume  g   pr(stDy rigume  gn (t wrongf (erro	( errrintf 2" %Di"eer ould nrigume  g-h
  f (erro	(ram 
   (e	ntf (}OST_NODE)
l printf (ntf (last_messag n
,"- %l)"s in SEde_file);f  e{eONLY__       er fiiiiiiiiiiiiram (
0 er fiiiiiiiiii printf (ntf (last_messag n
,"-)
aults"s in SEde_file);f  e{eONLY__)
aultsASE 2 fiiiiiiiiiiiiram (
0 er fiiiiiiiiii printf (ntf printf t
        ph>		/*  - %l)f (errors  pro
        ph>		/*  -)
aultsf (errors  proram (
   (errors printf (i++;  Nod }(e	essagrintf (" %l)",  nt({ro	(ram 
   (e	}(e	essaos (fn>= the f) sysrro%cat);
         spONLY__       
  prC)
        sprinC)
     >		/*    sprinC)
       [-h
   <dir> ]ntf (, "n de1.f");
   sprinC)
       [-sun]fiiiiiiiiiiii;
     ach enn/
UN4, eVM   sprinC)
       [-sgipvm]iiiiiiiiii;
     ach enn/
GI,  eVM   sprinC)
       [-ibm]fiiiiiiiiiiii;
     ach enn/
IBM, eVM   sprinC)
       [-ength-3]iiiiiiiii;
     ach enn/
Angth-3 FX/2800   sprinC)
       [-eng_pvm]iiiiiiiii;
     ach enn/
Angth-3 FX/2800t  exieVM   sprinC)
       [-ipsc]iiiiiiiiiiii;
     ach enn/
iPSC/86l   sprinC)
       [-gc]iiiiiiiiiiiiii;
     ach enn/
y coytec GC   sprinC)
       [-meiko_cs1]iiiiiii;
     ach enn/
Meiko CS 1   sprinC)
       [-meiko_cs2]iiiiiii;
     ach enn/
Meiko CS 2   sprinC)
       [-ksr]fiiiiiiiiiiii;
     ach enn/
KSR 1   sprinC)
       [-ksr_pvm]iiiiiiiii;
     ach enn/
KSR 1t  exieVM   sprinC)
       [-sgi]fiiiiiiiiiiii;
     ach enn/
iliconor.anlicsASIRIX)   sprinC)
       [-cm]iiiiiiiiiiiiii;
     ach enn/
CM 5   sprinC)
          sprinC)
       [-H | -N | -1]iiiii   }

a printf (,a    f = c,al Nodeod   sprinC)
       [-uniE 3 ]iiiiiiiii   }

al Nodeod, s  eras -1   sprinC)
       [-p <n>]iiiiiiiiiiiMinimalf ("\n" mpoE 3 : Aes   sprinC)
       [-F90|-F77]iiiiiiiiT
    L else /*(F;
    77tDy F;
    9l   sprinC)
       [-S");
|-D]iiiiiiiors in PHed/Dynam"node1.fs );
   Dy dynam"n   sprinC)
       [-split|-nosplit]iiSplit uprrors > HAE 2 : Cs   sprinC)
       [-ddr|-ddb|-ddcm];
 
aulteors in PHASErreplic> HA/block/CM   sprinC)
        sprinC)
       ( -nntf  | -sCGFile  | -cto  | -		/*  )  }
    f (erronC)
        spri}epFile ==_     "node1   spONLY__)
aultsASEy ( C)
        sprintC)
     D
aultsAmpor		/* :   sprintC)
     ===================   sprintC)
        sprintC)
     , "n de1.f");
iiiiiii:
      e  else /* tC)
     D
aulteors in PHASEr: generatessad)
ault_NODE //
DDEFAULT_REPLICATED)tC)
     replic> HAsad)r)   sprintessad)
ault_NODE //
DDEFAULT_DISTRIBUTEDEde_file)C)
     de1.fs rintblock drs in PHedsad)b)   sprintessad)
ault_NODE //
DDEFAULT_CM)tC)
     like
CM (ddcm)   sprintC)
     T
    Mach enn iiiii:
 sprintessa;
     ach enn//
ALLIANT)file)C)
     dngth-3 (Angth-3 FX/2800) sprintessa;
     ach enn//
ALL_PVM)file)C)
     dng_pvm (Angth-3 FX/2800, eVM) sprintessa;
     ach enn//
UN4_PVM)ile)C)
     sun (
UN4, eVM) sprintessa;
     ach enn//
OS2_PVM)file)C)
     os2pvm (OS2, eVM) sprintessa;
     ach enn//
RIOS_PVM)ile)C)
     ibmASIBM
RISC, eVM) sprintessa;
     ach enn//
IPSC_86l)ile)C)
     ipsc;
"PSC/86l) sprintessa;
     ach enn//
PARSYTEC_GC)tC)
       
(y coytec GCel) sprintessa;
     ach enn//
MEIKO_CS1)le)C)
     meiko_cs1 (Meiko CS1) sprintessa;
     ach enn//
MEIKO_CS2)le)C)
     meiko_cs2 (Meiko CS2) sprintessa;
     ach enn//
CM5)iiiiiiiiiC)
     Conn.f"ASErMach enn5 sprintessa;
     ach enn//
KSR1)iiiiiiiiC)
     Kendto  SqurintResearch 1 sprintessa;
     ach enn//
GI)iiiiiiiiiC)
     sgi (
iliconor.anlics) sprintC)
        sprintC)
     T
    M  }
 n iiiii:
 sprintessa;
     cgIllegal NodeodeiiiC)
  ("OennP 3 : ADy sprintessa;
     cgIllega    f = c)iiC)
  ("N=Only, nods sprintessa;
     cgIllega printf ("iiC)
  ("H= tar+ nods sprintC)
        sprintC)
     T
    L else /*iiiii:
 sprintessa;
      else /* target_mod77"iiC)
  ("F77*(F;
    77) sprintessa;
      else /* target_model =iC)
  ("F90*(F;
    90) sprintC)
          sprintC)
     Drs in PHed/Dynam"noAe1.fs rrorsbei:
 sprintessa ONLY_NODE */
        sprinde_fileC)
  ("S=);
     eximinimalfsoto fa d", ra3   exidagoto)printessa ONLY_NODE */
DYNAM     spri)eC)
  ("D=dynam"n sprintC)
        sprintC)
     Minimalf ("\n" mpoE 3 : Aes (p)iiii:
 ASE 6 rors);
)printC)
        sprintlast_plit_flagnde_file